==> fispro_3.4.orig.tar.gz: fispro-3.4/cpp/simple/fisimple.h <==
//***********************************************************************
//
//
//                              FISIMPLE.H
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
// File : FISIMPLE class description, used by FISPRO, 
//        part of package simple (Simplify a FIS)
//        requires library fispro
//
//**********************************************************************

#ifndef __FISIMPLE_H
#define __FISIMPLE_H

#include "fis.h"
#include "fpa.h"
#include <list>

//! Maximum number of Mf for an input (also defined in inclust.h) 
//! used to set the size of a group of rules.
#define MAX_MF_ARCHIVE 10  
//! Flag to undicate that none fusion could be achieved by TestGroupMerging
#define NO_FUSION     10026

//! Parameter for FISFPA
#define MIN 0
//! Parameter for FISFPA
#define DEC 1

//! Increment value of loss used in Complete 
#define STEP_LOSS 0.25
//! Increment value of heterogeneity used in Complete 
#define STEP_HET  0.25
//! Parameter used to compute the tolerated loss of performance (contructor)
#define RATE_ERR  0.02


//! Structure to handle a group of rules and to merge them into a generic rule
class GROUP 
//**********
{
  public :
    //! Rule numbers
    int r[MAX_MF_ARCHIVE];  
  //! Common (or generic) variable number
  int n;         
  //! Number of rules in the group      
  int eff;              
  //! True if the group is going to be merged
  int fusion;           
  //! Merging index value  
  double indice;   
  //! Conclusion value for the generic rule       
  double outval;
  //! Performance loss induced by this group merging
  double perfloss;

  int add(int v)
    {
      if(eff == MAX_MF_ARCHIVE) return false; // The array is full
      for(int i = 0; i < eff; i++)             
	if(v == r[i]) return true;            // v is already stored
      r[eff++] = v;                           // adding v
      return true;
    }

  void Print(FILE *f = stdout) const
    {
      int i;
      fprintf(f, "%3d\t%6.2f\t%3d\t", fusion, perfloss, n);
      for(i = 0; i < eff; i++) fprintf(f, "%4d ", r[i]);
      fprintf(f, "\n");
    }

  GROUP(int v1, int v2, int num)
    {
      n = num;
      r[0] = v1;
      r[1] = v2;
      eff = 2;
      for(int i = 2; i < MAX_MF_ARCHIVE; i++) r[i] = -1; 
      fusion = false;
      outval = -1.0;
      indice = 0.;
      perfloss = 0.0;
    }

  ~GROUP() {}
};



class FISIMPLE 
//************
{
  FISIMPLE& operator=(const FISIMPLE&);
  FISIMPLE(const FISIMPLE&);
  protected : 
    //! The FIS used for inference
    FIS * S;  
  int nbI, nbO;
  //sc=output homogeneity threshold-default value=0.8 (classif)/1000(inactive)
  double sc, se, pabs;
  //! The temporary rule base
  RULE ** Save;
  //! The number of rules in Save
  int NbSave;     
  //! The FIS config file name
  char * fConfig;    
  //! The training set file name
  char * fApp;  
  //! The validation set file name
  char * fTest;  
  //! The result (log) file name
  char * fResult;   
  //!  The most simple FIS file name
  char * fSimple;   
  //! Number of items in the training set
  int NbEx;          
  //! Number of items in the validation set
  int NbTestEx;          
  //! Output number
  int NSort;    
  //! Output standard deviation computed from the training set 
  //! (if not classification)     
  double StdDev; 

  int Display;

  int RuleRemove;

  int VarRemove;

  //! Activity threshold for an item not to be blank
  double MuMin;

  //! Tolerated loss of coverage (and the flag when coverage threshold is overcome)
  double PropBlank; 
  double InitCov, CovLoss;
  //! Init, current and maximum performance indices
  double InitPi, CurPi, MaxPi, CurCov;    
  //! Iteration counter. Used to form archive file names
  int it;  
  //! The training data array 
  double ** Data;
  //! The validation data array 
  double ** vData;

  //! Do not remove the last rule with a given class label as conclusion. 
  //! Used only in a crisp classification case
  int KeepLast;
  //! True when output crisp and classification type
  int ClassifType;
  //! Number of classes or output MF (used with KeepLast)
  int Cla;
  //! For each class or output MF (init from data) this array contains 
  //! the number of rules whose conclusion is the class or the MF label
  int * RuleClass;
  //! The misclassification result array 
  int * MisClass;
  //! The class or MF labels array
  double * RuleLab;
  //! Reset the RuleClass array according to the current rule base 
  void ResetRuleClass(void);
 
  list <GROUP *> G;
  int  CreateGroups(int relax);
  list <GROUP *>::iterator ExistingGroup(int r, int var);
  //  int  TestGroupMerging(void); 
  //! 'het' is the proportion of heterogeneity tolerated
  int TestGroupMerging(double het); 
  int RemoveUnvalidGroups(void);
  int OrderGroupsByPi(void);
  int MakeValidMergings(double loss);
  void RemoveGroups(void);
  void PrintGroups(FILE *f = stdout);
  //! Return 1 if there is only one value different from ANY in
  //! the premise else 0.
  int BuildRule(GROUP * g, int &OldInput, double &OldOutput);
  void RestoreGroup(GROUP * g, int valI, double valO);
  //! Used within the KeepLast framework. 
  //! Return true if one of the rules of the group is the last 
  //! with a given class conclusion.
  int IsLast(GROUP * g);

  //! Used within the KeepLast framework. 
  //! Update the RuleClass array during the group merging procedure.
  void UpdateRuleClass(GROUP * g);


  //! Return true if all the premise propositions, except the number 'f', 
  //! are set to ANY
  int AllAny(RULE *R, int f);

  //! If rule 'R' is already stored in Save return its number, 
  //! else return -1.
  int Exists(RULE *R);

  //! Record the string corresponding to rule 'R' in 'str'. 
  //! 'str' may be used as argument for RULE constructor. 
  //! 'str' must be previously allocated
  void RecString(RULE * R, char * str);

  void UpdateSave(list <GROUP *>::iterator begin,list <GROUP *>::iterator end);
  //! Reset the temporary rule base 'Save' from the current configuration of FIS 'S'
  int  ResetSave(void);
  //! Store the current configuration of 'Save' and the structure of 'S' into a FIS 
  //! configuration file (file name made up from 'fConfig' and 'i') and reset 'S' 
  //! with this configuration.
  void WriteFis(int i);

  //! Update 'S' and 'Save' according to the configuration stored in the corresponding 
  //! file (file name made up from 'fConfig' and 'i') and reset 'S' 
  void ReadFis(int i);    

  //! Call FIS::Performance with 'S' and the validation file name. 
  //! Return the variation of the performance index computed from 'PerfRef', 
  //! reference, and 'PerfCur', the current value given by Performance.
  //! 'Cov' is the coverage level.
  double ComputePI(double PerfRef, double & PerfCur, double & Cov, double & Err);

  //! Store the successful attempt of simplification in the result file 
  //! name 'fic' ('fResult'). 'PerfCur', 'Cov' and 'Err' are calculated 
  //! using the ComputePI function.
  void StoreResult(double PerfCur, double Cov, double Err, char * fic);
  
  //! Compute initial performance and coverage.
  //! Function called within the constructor and by InitTestData
  void InitIndices(void);

  //! Open the training data file and allocate and fill 'RuleLab', 
  //! 'RuleClass' and 'MisClass' arrays, fill the 'StdDev', 'NbEx', 
  //! 'Cla' and 'KeepLast' fields.
  void InitData(void);  
      

  public :

  FISIMPLE(char *ffis, char *fdat, char *fres, double c, double e, int ns, double mmin, double pblank, double cov, int kl = 0, double me = -1.);

  ~FISIMPLE() 
    {
      delete [] fConfig;
      delete [] fResult;
      delete [] fSimple;
      // correction leak memory - bch July 20 2009
      if (Save !=NULL)
	for(int i = 0; i < NbSave; i++)    
	  delete Save[i];
      delete [] Save;
      Save=NULL;
      if(MisClass) delete [] MisClass;
      if(RuleClass) delete [] RuleClass;
      if(RuleLab) delete [] RuleLab;
      // correction leak memory - bch July 20 2009
      /*for(list<GROUP *>::iterator p=G.begin();p!=G.end();p++)
	{
	  (*p)->Print();
	  delete (*p);
	  }*/
      delete S;

      if(Data != NULL) 
	{
	  // correction bug delete -> delete [] - bch July 20 2009
	  for(int i = 0; i < NbEx; i++)  delete [] Data[i];
	  delete [] Data;
	  Data=NULL;
	}

      if(fTest != NULL) 
	{
	  for(int i = 0; i < NbTestEx; i++)  delete [] vData[i];
	  delete [] vData;
	  vData=NULL;
	}
    
    }

  //! This variable indicates the criterion which stopped the 
  //! simplification process. 
  //! Heterogeneity : 2, performance : 3, No more merging possibilities : 4. 
  int StopCrit;

  //! Remove all temporary files including fis files, 
  //! except the one corresponding to string 's'.
  void RemoveTmpFiles(char * s);

  //! Write the current FIS 'S' with the active rules of 'Save' in file 'f'. 
  //! Function called by WriteFis.  
  void PrintCfgFis(FILE * f); 

  //! Deactivate inputs whose number of MF is only one (universal set), 
  //! set the corresponding label to ANY in all the rules and write 
  //! the configuration in the file "'fConfig'.0"
  void Prepare(void);

  //! The most important level of simplification:  merging of groups of rules 
  //! into generic rules. 'loss' is the tolerated loss of performance. 
  //! Return:  number of the iteration to be done, 'it'.
  int GroupMerging(double loss, double het);

  //! The rule level of the simplication procedure. Rule removal is done according 
  //! to performance and coverage indices. 'loss' is the tolerated loss of performance. 
  //! Return:  number of the iteration to be done, 'it'.
  int RuleRemoval(double loss);

  //! The final level of the simplication procedure. Variable removal is
  //! studied within each isolated rule, according to performance and coverage indices. 
  //! 'loss' is the tolerated loss of performance. 
  //! Return:  number of the iteration to be done, 'it'.
  int VariableRemoval(double loss);

  //! A smart combination of the three levels of simplification. Call of the functions 
  //! GroupMerging, RuleRemoval, VariableRemoval with a priority to GroupMerging. 
  //! These functions are called with increased values of toleraated loss of performance. 
  //! Return:  a pointer on the name of the simplest FIS configuration file name 
  //! which satisfies all the given criteria of performance and coverage.
  char * Complete(bool wl=false);

  //! Open the validation data file, fill the vData array and call InitIndices
  void InitTestData(char *TestData);
  void SetDisplay(int num){Display=num;}
  int GetDisplay(void) const {return Display;}
  void SetRuleRemove(int num){RuleRemove=num;}
  int GetRuleRemove(void) const {return RuleRemove;}
  void SetVarRemove(int num){VarRemove=num;}
  int GetVarRemove(void) const {return VarRemove;} 
};

#endif
//*********************   End of FISIMPLE.H      ***********************

==> fispro_3.6.orig.tar.gz: fispro-3.6/cpp/simple/fisimple.h <==
//***********************************************************************
//
//
//                              FISIMPLE.H
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
// File : FISIMPLE class description, used by FISPRO, 
//        part of package simple (Simplify a FIS)
//        requires library fispro
//
//**********************************************************************

#ifndef __FISIMPLE_H
#define __FISIMPLE_H

#include "fis.h"
#include "fpa.h"
#include <list>

//! Maximum number of Mf for an input (also defined in inclust.h) 
//! used to set the size of a group of rules.
#define MAX_MF_ARCHIVE 10  
//! Flag to undicate that none fusion could be achieved by TestGroupMerging
#define NO_FUSION     10026

//! Parameter for FISFPA
#define MIN 0
//! Parameter for FISFPA
#define DEC 1

//! Increment value of loss used in Complete 
#define STEP_LOSS 0.25
//! Increment value of heterogeneity used in Complete 
#define STEP_HET  0.25
//! Parameter used to compute the tolerated loss of performance (contructor)
#define RATE_ERR  0.02


//! Structure to handle a group of rules and to merge them into a generic rule
class GROUP 
//**********
{
  public :
    //! Rule numbers
    int r[MAX_MF_ARCHIVE];  
  //! Common (or generic) variable number
  int n;         
  //! Number of rules in the group      
  int eff;              
  //! True if the group is going to be merged
  int fusion;           
  //! Merging index value  
  double indice;   
  //! Conclusion value for the generic rule       
  double outval;
  //! Performance loss induced by this group merging
  double perfloss;

  int add(int v)
    {
      if(eff == MAX_MF_ARCHIVE) return false; // The array is full
      for(int i = 0; i < eff; i++)             
	if(v == r[i]) return true;            // v is already stored
      r[eff++] = v;                           // adding v
      return true;
    }

  void Print(FILE *f = stdout) const
    {
      int i;
      fprintf(f, "%3d\t%6.2f\t%3d\t", fusion, perfloss, n);
      for(i = 0; i < eff; i++) fprintf(f, "%4d ", r[i]);
      fprintf(f, "\n");
    }

  GROUP(int v1, int v2, int num)
    {
      n = num;
      r[0] = v1;
      r[1] = v2;
      eff = 2;
      for(int i = 2; i < MAX_MF_ARCHIVE; i++) r[i] = -1; 
      fusion = false;
      outval = -1.0;
      indice = 0.;
      perfloss = 0.0;
    }

  ~GROUP() {}
};



class FISIMPLE 
//************
{
  FISIMPLE& operator=(const FISIMPLE&);
  FISIMPLE(const FISIMPLE&);
  protected : 
    //! The FIS used for inference
    FIS * S;  
  int nbI, nbO;
  //sc=output homogeneity threshold-default value=0.8 (classif)/1000(inactive)
  double sc, se, pabs;
  //! The temporary rule base
  RULE ** Save;
  //! The number of rules in Save
  int NbSave;     
  //! The FIS config file name
  char * fConfig;    
  //! The training set file name
  const char * fApp;
  //! The validation set file name
  char * fTest;  
  //! The result (log) file name
  char * fResult;   
  //!  The most simple FIS file name
  char * fSimple;   
  //! Number of items in the training set
  int NbEx;          
  //! Number of items in the validation set
  int NbTestEx;          
  //! Output number
  int NSort;    
  //! Output standard deviation computed from the training set 
  //! (if not classification)     
  double StdDev; 

  int Display;

  int RuleRemove;

  int VarRemove;

  //! Activity threshold for an item not to be blank
  double MuMin;

  //! Tolerated loss of coverage (and the flag when coverage threshold is overcome)
  double PropBlank; 
  double InitCov, CovLoss;
  //! Init, current and maximum performance indices
  double InitPi, CurPi, MaxPi, CurCov;    
  //! Iteration counter. Used to form archive file names
  int it;  
  //! The training data array 
  double ** Data;
  //! The validation data array 
  double ** vData;

  //! Do not remove the last rule with a given class label as conclusion. 
  //! Used only in a crisp classification case
  int KeepLast;
  //! True when output crisp and classification type
  int ClassifType;
  //! Number of classes or output MF (used with KeepLast)
  int Cla;
  //! For each class or output MF (init from data) this array contains 
  //! the number of rules whose conclusion is the class or the MF label
  int * RuleClass;
  //! The misclassification result array 
  int * MisClass;
  //! The class or MF labels array
  double * RuleLab;
  //! Reset the RuleClass array according to the current rule base 
  void ResetRuleClass(void);
 
  list <GROUP *> G;
  int  CreateGroups(int relax);
  list <GROUP *>::iterator ExistingGroup(int r, int var);
  //  int  TestGroupMerging(void); 
  //! 'het' is the proportion of heterogeneity tolerated
  int TestGroupMerging(double het); 
  int RemoveUnvalidGroups(void);
  int OrderGroupsByPi(void);
  int MakeValidMergings(double loss);
  void RemoveGroups(void);
  void PrintGroups(FILE *f = stdout);
  //! Return 1 if there is only one value different from ANY in
  //! the premise else 0.
  int BuildRule(GROUP * g, int &OldInput, double &OldOutput);
  void RestoreGroup(GROUP * g, int valI, double valO);
  //! Used within the KeepLast framework. 
  //! Return true if one of the rules of the group is the last 
  //! with a given class conclusion.
  int IsLast(GROUP * g);

  //! Used within the KeepLast framework. 
  //! Update the RuleClass array during the group merging procedure.
  void UpdateRuleClass(GROUP * g);


  //! Return true if all the premise propositions, except the number 'f', 
  //! are set to ANY
  int AllAny(RULE *R, int f);

  //! If rule 'R' is already stored in Save return its number, 
  //! else return -1.
  int Exists(RULE *R);

  //! Record the string corresponding to rule 'R' in 'str'. 
  //! 'str' may be used as argument for RULE constructor. 
  //! 'str' must be previously allocated
  void RecString(RULE * R, char * str);

  void UpdateSave(list <GROUP *>::iterator begin,list <GROUP *>::iterator end);
  //! Reset the temporary rule base 'Save' from the current configuration of FIS 'S'
  int  ResetSave(void);
  //! Store the current configuration of 'Save' and the structure of 'S' into a FIS 
  //! configuration file (file name made up from 'fConfig' and 'i') and reset 'S' 
  //! with this configuration.
  void WriteFis(int i);

  //! Update 'S' and 'Save' according to the configuration stored in the corresponding 
  //! file (file name made up from 'fConfig' and 'i') and reset 'S' 
  void ReadFis(int i);    

  //! Call FIS::Performance with 'S' and the validation file name. 
  //! Return the variation of the performance index computed from 'PerfRef', 
  //! reference, and 'PerfCur', the current value given by Performance.
  //! 'Cov' is the coverage level.
  double ComputePI(double PerfRef, double & PerfCur, double & Cov, double & Err);

  //! Store the successful attempt of simplification in the result file 
  //! name 'fic' ('fResult'). 'PerfCur', 'Cov' and 'Err' are calculated 
  //! using the ComputePI function.
  void StoreResult(double PerfCur, double Cov, double Err, char * fic);
  
  //! Compute initial performance and coverage.
  //! Function called within the constructor and by InitTestData
  void InitIndices(void);

  //! Open the training data file and allocate and fill 'RuleLab', 
  //! 'RuleClass' and 'MisClass' arrays, fill the 'StdDev', 'NbEx', 
  //! 'Cla' and 'KeepLast' fields.
  void InitData(void);  
      

  public :

  FISIMPLE(const char *ffis, const char *fdat, const char *fres, double c, double e, int ns, double mmin, double pblank, double cov, int kl = 0, double me = -1.);

  ~FISIMPLE() 
    {
      delete [] fConfig;
      delete [] fResult;
      delete [] fSimple;
      // correction leak memory - bch July 20 2009
      if (Save !=NULL)
	for(int i = 0; i < NbSave; i++)    
	  delete Save[i];
      delete [] Save;
      Save=NULL;
      if(MisClass) delete [] MisClass;
      if(RuleClass) delete [] RuleClass;
      if(RuleLab) delete [] RuleLab;
      // correction leak memory - bch July 20 2009
      /*for(list<GROUP *>::iterator p=G.begin();p!=G.end();p++)
	{
	  (*p)->Print();
	  delete (*p);
	  }*/
      delete S;

      if(Data != NULL) 
	{
	  // correction bug delete -> delete [] - bch July 20 2009
	  for(int i = 0; i < NbEx; i++)  delete [] Data[i];
	  delete [] Data;
	  Data=NULL;
	}

      if(fTest != NULL) 
	{
	  for(int i = 0; i < NbTestEx; i++)  delete [] vData[i];
	  delete [] vData;
	  vData=NULL;
	}
    
    }

  //! This variable indicates the criterion which stopped the 
  //! simplification process. 
  //! Heterogeneity : 2, performance : 3, No more merging possibilities : 4. 
  int StopCrit;

  //! Remove all temporary files including fis files, 
  //! except the one corresponding to string 's'.
  void RemoveTmpFiles(char * s);

  //! Write the current FIS 'S' with the active rules of 'Save' in file 'f'. 
  //! Function called by WriteFis.  
  void PrintCfgFis(FILE * f); 

  //! Deactivate inputs whose number of MF is only one (universal set), 
  //! set the corresponding label to ANY in all the rules and write 
  //! the configuration in the file "'fConfig'.0"
  void Prepare(void);

  //! The most important level of simplification:  merging of groups of rules 
  //! into generic rules. 'loss' is the tolerated loss of performance. 
  //! Return:  number of the iteration to be done, 'it'.
  int GroupMerging(double loss, double het);

  //! The rule level of the simplication procedure. Rule removal is done according 
  //! to performance and coverage indices. 'loss' is the tolerated loss of performance. 
  //! Return:  number of the iteration to be done, 'it'.
  int RuleRemoval(double loss);

  //! The final level of the simplication procedure. Variable removal is
  //! studied within each isolated rule, according to performance and coverage indices. 
  //! 'loss' is the tolerated loss of performance. 
  //! Return:  number of the iteration to be done, 'it'.
  int VariableRemoval(double loss);

  //! A smart combination of the three levels of simplification. Call of the functions 
  //! GroupMerging, RuleRemoval, VariableRemoval with a priority to GroupMerging. 
  //! These functions are called with increased values of toleraated loss of performance. 
  //! Return:  a pointer on the name of the simplest FIS configuration file name 
  //! which satisfies all the given criteria of performance and coverage.
  char * Complete(bool wl=false);

  //! Open the validation data file, fill the vData array and call InitIndices
  void InitTestData(char *TestData);
  void SetDisplay(int num){Display=num;}
  int GetDisplay(void) const {return Display;}
  void SetRuleRemove(int num){RuleRemove=num;}
  int GetRuleRemove(void) const {return RuleRemove;}
  void SetVarRemove(int num){VarRemove=num;}
  int GetVarRemove(void) const {return VarRemove;} 
};

#endif
//*********************   End of FISIMPLE.H      ***********************
